// StringVariable Implentation File
// File StringVariable.h

#include <iostream.h>
#include <string.h>
#include "strvar.h"


// Constructor
StringVariable::StringVariable (const char *s)
: StringBase (s) 
{
	// ** Very Important to call StringBase Constructor **
}


// Destructor
StringVariable::~StringVariable ()
{
	// delete is done in base class StingBase ??
}


// assigning a character string
StringVariable &StringVariable::operator= (const char *other)
{		
	if (m_string)
	  delete [] m_string;

	m_length = strlen(other);
	m_string = new char [m_length + 1];
	strcpy (m_string, other);

	return *this;
}


// Assignment Operator to another String object
StringVariable &StringVariable::operator= (const StringVariable &other)
{	
	if (this == &other)				// "other" is reference to another of StringVariable object
	  return *this;						// take care of self-assignment such as object R=R

	if (m_string) delete [] m_string;				// in case of non-existence of "m_string"

	m_length = other.m_length;						  // assignment to new string length
	char *temp = new char [m_length + 1];   // create dynamic memory allocation
	strcpy (temp, other.m_string);
	m_string = temp;												// switch pointer	

	return *this;
}


// overloading an equality operator with another "object"
StringVariable::operator== (const StringVariable &other) const
{
	// return (strcmp(m_string, other.m_string) == 0 ? TRUE : FALSE);
	
	if (strcmp(m_string, other.m_string) == 0)
		return TRUE;
	else
		return FALSE;
}


// overloading equality operator with another "string"
BOOL StringVariable::operator== (const char *other) const
{
	// return (strcmp(m_string, other.m_string) == 0 ? TRUE : FALSE);
	
	if (strcmp(m_string, other) == 0)
		return TRUE;
	else
		return FALSE;
}




// Copy Operator Overloading
StringVariable::StringVariable(const StringVariable &other)
{
	// delete of "m_string" is not neceesary
	m_length = other.m_length;
	m_string =  new char [m_length + 1];		// no need for char *m_string because it 
																					// is already defined in class/private
	strcpy (m_string, other.m_string);
}


// Append function with another StringVariable object
StringVariable &StringVariable::Append (const StringVariable &other)
{
	m_length += other.m_length;
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other.m_string);

	delete [] m_string;

	m_string = temp;

	return *this;
}


// Append function with another string
StringVariable &StringVariable::Append (const char *other)
{
	m_length += strlen(other);
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other);

	delete [] m_string;

	m_string = temp;

	return *this;
}


// Overloading Concatenation operator with another string
StringVariable &StringVariable::operator+= (const char *other)
{
	m_length += strlen(other);
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other);

	delete [] m_string;

	m_string = temp;

	return *this;
}


// Overloading Concatenation operator with another StringVariable object
StringVariable &StringVariable::operator+= (const StringVariable &other)
{
	m_length += other.m_length;
	char *temp = new char [m_length + 1];

	strcpy (temp, m_string);
	strcat (temp, other.m_string);

	delete [] m_string;

	m_string = temp;

	return *this;
}


// function to display substrings
void StringVariable::Display_Substring (const int Start, const int End) const
{
	if (End < Start || Start < 1 || End > m_length)
		return;

	for (int i = Start; i <= End; i++) 	
	{
		cout << m_string[i-1];
	}
}


// overloading ! operator
BOOL StringVariable::operator! (void) const
{
	return (m_length==0);  // returning TURE = empty
}

